  // INCLUDE WIRE
  #include <Wire.h>

  // Custom I2C PINS FOR ESP32C3
  // #define I2C_SDA_PIN 7
  // #define I2C_SCL_PIN 6
  // #define I2C_SDA_PIN 8
  // #define I2C_SCL_PIN 9

  // LIST OF COMMANDS
  #define COMMAND_DICE        0x01
  #define COMMAND_UPDATE      0x02
  #define COMMAND_PERMISSION  0x03
  #define COMMAND_DENY        0x04

  // PLAYER
  struct _player {
    uint8_t x;
    uint8_t y;
    uint8_t h;
  };

  // MAP STATUS
  struct _map {
    _player red;
    _player blue;
  };

  // MAP INSTANCE
  _map  MAP;

//============================================================
//
//============================================================

  void setup() {
    Serial.begin(115200);
    // Wire.setPins(I2C_SDA_PIN, I2C_SCL_PIN);
    Wire.begin();
    delay(100);
    Serial.println("Master ready. Scanning for slaves...");
    scanI2C();
  }

//============================================================
//
//============================================================

  void loop() {


    Wire.beginTransmission(0x10);
    Wire.write(2);
    Wire.endTransmission();

    delay(1000);

    // Questions For You
    // uint8_t address = Question("Joystic Address ").toInt();
    // uint8_t command = Question("Command Code    ").toInt();

    // // Select Command You Where Looking For
    // switch(command) {
    //   case 1: Dice(address); break;
    //   case 2: Update(address); break;
    //   case 3: Permission(address); break;
    //   case 4: Deny(address); break;
    // }

  }

//============================================================
//
//============================================================

  void scanI2C() {
    byte error, address;
    int nDevices = 0;
    Serial.println("Scanning I2C bus...");
    for(address = 1; address < 127; address++) {
      Wire.beginTransmission(address);
      error = Wire.endTransmission();
      if (error == 0) {
        Serial.print("I2C device found at address 0x");
        Serial.println(address, HEX);
        nDevices++;
      }
    }
    if (nDevices == 0)
      Serial.println("No I2C devices found");
  }

//============================================================
//
//============================================================

  void send_i2c_data(uint8_t address, uint8_t command, uint8_t *data) {

    Serial.print  ("Address : 0x");
    Serial.println(address, HEX);    
    Wire.beginTransmission(address);
    
    Serial.print  ("Command : 0x");
    Serial.println(command, HEX);
    Wire.write(command);

    size_t length = sizeof(data);

    Serial.print  ("Data    : ");
    
    for(int i=0; i<=length; i++) {
      Serial.print("0x");
      Serial.print(data[i], HEX);
      Wire.write(data[i]);
      Serial.print(", ");
    }

    Serial.println();
    Wire.endTransmission();

  }

//============================================================
//
//============================================================

  uint8_t Dice(uint8_t address) {

    // Print Data On Serial - Display Address
    Serial.print("Address : 0x"); Serial.println(address, HEX);
    
    // Print Data On Serial - Display Command
    Serial.print("Command : 0x"); Serial.println(COMMAND_DICE, HEX);
    
    // Sending The Command
    Wire.beginTransmission(address);    // Sending To Slave Address
    Wire.write(COMMAND_DICE);           // Sending Command - DiceRoll
    Wire.endTransmission();             // Stop Transmission

    // Request Data From Slave
    Wire.requestFrom(address, 1);       // request 1 byte from slave

    // Dice Value
    uint8_t value;

    // Method One
    while(!Wire.available());
    value = Wire.read();

    // Method Two
    // if(Wire.available()) {
    //   value = Wire.read();
    // }

    // Stop Transmission
    Wire.endTransmission();

    // Print Value And Return Value
    Serial.println("Dice Value : " + String(value));

    // Return Value
    return value;

  }

//============================================================
//
//============================================================

  void Update(uint8_t address) {

    // Print Data On Serial - Display Address
    Serial.print("Address : 0x"); Serial.println(address, HEX);
    
    // Print Data On Serial - Display Command
    Serial.print("Command : 0x"); Serial.println(COMMAND_UPDATE, HEX);
    
    // Sending The Command
    Wire.beginTransmission(address);    // Sending To Slave Address
    Wire.write(COMMAND_DICE);           // Sending Command - DiceRoll
    Wire.write(MAP.red.x);              // Sending RED x Position
    Wire.write(MAP.red.y);              // Sending RED y Position
    Wire.write(MAP.red.h);              // Sending RED Health
    Wire.write(MAP.blue.x);             // Sending BLUE x Position
    Wire.write(MAP.blue.y);             // Sending BLUE y Position
    Wire.write(MAP.blue.h);             // Sending BLUE Health
    Wire.endTransmission();             // Stop Transmission

  }

//============================================================
//
//============================================================

  uint8_t* Permission(uint8_t address) {

    // Print Data On Serial - Display Address
    Serial.print("Address : 0x"); Serial.println(address, HEX);
    
    // Print Data On Serial - Display Command
    Serial.print("Command : 0x"); Serial.println(COMMAND_PERMISSION, HEX);
    
    // Sending The Command
    Wire.beginTransmission(address);    // Sending To Slave Address
    Wire.write(COMMAND_PERMISSION);     // Sending Command - Permission
    Wire.endTransmission();             // Stop Transmission

    // Request Data From Slave
    Wire.requestFrom(address, 7);       // Request 7 Byte From Slave

    // Dice Value
    uint8_t value[6];

    // Method One
    // while(!Wire.available());
    // value = Wire.read();

    // Print Value And Return Value
    Serial.println("Dice Value : ");

    // Method Two
    if(Wire.available()) {
      for(int i=0; i<6; i++) {
        value[i] = Wire.read();
        Serial.println("  Value : " + String(value[i]));
      }
    }

    // Stop Transmission
    Wire.endTransmission();

    // Return Value
    return value;

  }

//============================================================
//
//============================================================

  void Deny(uint8_t address) {

    Serial.print  ("Address : 0x"); Serial.println(address, HEX);    
    Serial.print("Command : 0x"); Serial.println(COMMAND_DENY, HEX);

    Wire.beginTransmission(address);
    Wire.write(COMMAND_DENY);
    Wire.endTransmission();

  }

//============================================================
//
//============================================================

  String Question(String Msg) {

    Serial.print(Msg + " : ");
    while(!Serial.available());
    Msg = Serial.readStringUntil('\n');
    while(Serial.available()) { Serial.read(); }
    Msg.trim();
    Serial.println(Msg);
    return Msg;

  }

//============================================================
//
//============================================================